@@ -4,9 +4,8 @@
 from string import Template
 
 from sql import Literal
-from sql.conditionals import Coalesce
 from sql.functions import Substring
-from sql.operators import Concat, Equal
+from sql.operators import Equal
 
 from trytond.cache import Cache
 from trytond.i18n import gettext
@@ -63,7 +62,7 @@ class Address(
     identifiers = fields.One2Many(
         'party.identifier', 'address', "Identifiers",
         domain=[
-            ('party', '=', Eval('party')),
+            ('party', '=', Eval('party', -1)),
             ('type', 'in', ['fr_siret']),
             ])
     contact_mechanisms = fields.One2Many(
@@ -83,8 +82,6 @@ class Address(
 
     @classmethod
     def __register__(cls, module_name):
-        cursor = Transaction().connection.cursor()
-        sql_table = cls.__table__()
         table = cls.__table_handler__(module_name)
 
         # Migration from 5.8: rename zip to postal code
@@ -92,18 +89,6 @@ class Address(
 
         super(Address, cls).__register__(module_name)
 
-        table = cls.__table_handler__(module_name)
-
-        # Migration from 4.0: remove streetbis
-        if table.column_exist('streetbis'):
-            value = Concat(
-                Coalesce(sql_table.street, ''),
-                Concat('\n', Coalesce(sql_table.streetbis, '')))
-            cursor.execute(*sql_table.update(
-                    [sql_table.street],
-                    [value]))
-            table.drop_column('streetbis')
-
     @fields.depends('street')
     def on_change_with_street_single_line(self, name=None):
         if self.street:
@@ -139,14 +124,20 @@ class Address(
 
     @fields.depends('city', methods=['_autocomplete_domain'])
     def autocomplete_postal_code(self):
-        domain = self._autocomplete_domain()
+        domain = [
+            self._autocomplete_domain(),
+            ('postal_code', 'not in', [None, '']),
+            ]
         if self.city:
             domain.append(('city', 'ilike', '%%%s%%' % self.city))
         return self._autocomplete_search(domain, 'postal_code')
 
     @fields.depends('postal_code', methods=['_autocomplete_domain'])
     def autocomplete_city(self):
-        domain = self._autocomplete_domain()
+        domain = [
+            self._autocomplete_domain(),
+            ('city', 'not in', [None, '']),
+            ]
         if self.postal_code:
             domain.append(('postal_code', 'ilike', '%s%%' % self.postal_code))
         return self._autocomplete_search(domain, 'city')
@@ -265,7 +256,8 @@ class Address(
     def get_subdivision_types(cls):
         pool = Pool()
         Subdivision = pool.get('country.subdivision')
-        return Subdivision.fields_get(['type'])['type']['selection']
+        selection = Subdivision.fields_get(['type'])['type']['selection']
+        return [(k, v) for k, v in selection if k is not None]
 
     @fields.depends('country')
     def on_change_with_subdivision_types(self, name=None):
@@ -437,7 +429,8 @@ class SubdivisionType(DeactivableMixin, ModelSQL, ModelView):
     def get_subdivision_types(cls):
         pool = Pool()
         Subdivision = pool.get('country.subdivision')
-        return Subdivision.fields_get(['type'])['type']['selection']
+        selection = Subdivision.fields_get(['type'])['type']['selection']
+        return [(k, v) for k, v in selection if k is not None]
 
     @classmethod
     def create(cls, *args, **kwargs):
