@@ -84,7 +84,7 @@ class Party(
         fields.Char("Fax", states=_contact_mechanism_states),
         'get_contact_mechanism', setter='set_contact_mechanism')
     email = fields.Function(
-        fields.Char("E-Mail", states=_contact_mechanism_states),
+        fields.Char("Email", states=_contact_mechanism_states),
         'get_contact_mechanism', setter='set_contact_mechanism')
     website = fields.Function(
         fields.Char("Website", states=_contact_mechanism_states),
@@ -102,8 +102,8 @@ class Party(
             ('code_uniq', Unique(t, t.code), 'party.msg_party_code_unique')
             ]
         cls._sql_indexes.update({
-                Index(t, (t.code, Index.Equality())),
-                Index(t, (t.code, Index.Similarity())),
+                Index(t, (t.code, Index.Equality(cardinality='high'))),
+                Index(t, (t.code, Index.Similarity(cardinality='high'))),
                 })
         cls._order.insert(0, ('distance', 'ASC NULLS LAST'))
         cls._order.insert(1, ('name', 'ASC'))
@@ -280,21 +280,25 @@ class Party(
         return index
 
     @classmethod
-    def _new_code(cls, **pattern):
+    def _code_sequence(cls, **pattern):
         pool = Pool()
         Configuration = pool.get('party.configuration')
         config = Configuration(1)
-        sequence = config.get_multivalue('party_sequence', **pattern)
-        if sequence:
-            return sequence.get()
+        return config.get_multivalue('party_sequence', **pattern)
 
     @classmethod
     def create(cls, vlist):
         vlist = [x.copy() for x in vlist]
+        missing_code = []
         for values in vlist:
             if not values.get('code'):
-                values['code'] = cls._new_code()
+                missing_code.append(values)
             values.setdefault('addresses', None)
+        if missing_code:
+            if sequence := cls._code_sequence():
+                for values, code in zip(
+                        missing_code, sequence.get_many(len(missing_code))):
+                    values['code'] = code
         return super(Party, cls).create(vlist)
 
     @classmethod
@@ -600,6 +604,7 @@ IDENTIFIER_TYPES = [
     ('gb_nhs',
         "United Kingdom National Health Service Patient Identifier"),
     ('gb_upn', "English Unique Pupil Number"),
+    ('gb_utr', "United Kingdom Unique Taxpayer Reference"),
     ('gb_vat', "United Kingdom (and Isle of Man) VAT Number"),
     ('gh_tin', "Ghanaian Taxpayer Identification Number"),
     ('gn_nifp', "Guinean Tax Number"),
@@ -623,6 +628,7 @@ IDENTIFIER_TYPES = [
     ('it_codicefiscale', "Italian Tax Code for Individuals"),
     ('it_iva', "Italian VAT Number"),
     ('jp_cn', "Japanese Corporate Number"),
+    ('ke_pin', "Kenya Tax Number"),
     ('kr_brn', "South Korea Business Registration Number"),
     ('kr_krn', "South Korean Resident Registration Number"),
     ('lt_asmens', "Lithuanian Personal Number"),
@@ -632,6 +638,7 @@ IDENTIFIER_TYPES = [
     ('ma_ice', "Moroccan Tax Number"),
     ('mc_tva', "Monacan VAT Number"),
     ('md_idno', "Moldavian Company Identification Number"),
+    ('me_pib', "Montenegro Tax Number"),
     ('mk_edb', "Macedonian Tax Number"),
     ('mt_vat', "Maltese VAT Number"),
     ('mu_nid', "Mauritian National Identifier"),
@@ -664,17 +671,21 @@ IDENTIFIER_TYPES = [
     ('se_orgnr', "Swedish Company Number"),
     ('se_personnummer', "Swedish Personal Number"),
     ('se_vat', "Swedish VAT Number"),
+    ('sg_uen', "Singapore's Unique Entity Number"),
     ('si_businessid', "Slovenian Corporate Registration Number"),
     ('si_ddv', "Slovenian VAT Number"),
     ('si_emso', "Slovenian Unique Master Citizen Number"),
     ('sk_dph', "Slovak VAT Number"),
     ('sk_rc', "Slovak Birth Number"),
     ('sm_coe', "San Marino National Tax Number"),
+    ('sv_nit', "El Salvador Tax Number"),
     ('th_moa', "Thai Memorandum of Association Number"),
     ('th_pin', "Thai Personal Identification Number"),
     ('th_tin', "Thai Taxpayer Identification Number"),
     ('tn_mf', "Tunisian Tax Number"),
+    ('tr_vkn', "Turkish Tax Identification Number"),
     ('tr_tckimlik', "Turkish Personal Identification Number"),
+    ('tw_ubn', "Taiwanese Tax Number"),
     ('ua_edrpou', "Ukrainian Identifier for Enterprises and Organizations"),
     ('ua_rntrc', "Ukrainian Individual Taxpayer Registration Number"),
     ('us_atin', "U.S. Adoption Taxpayer Identification Number"),
@@ -683,7 +694,7 @@ IDENTIFIER_TYPES = [
     ('us_ptin', "U.S. Preparer Tax Identification Number"),
     ('us_ssn', "U.S. Social Security Number"),
     ('us_tin', "U.S. Taxpayer Identification Number"),
-    ('uy_ruc', "Uruguay Tax Number"),
+    ('uy_rut', "Uruguay Tax Number"),
     ('ve_rif', "Venezuelan VAT Number"),
     ('vn_mst', "Vietnam Tax Number"),
     ('za_idnr', "South African Identity Document Number"),
@@ -735,6 +746,7 @@ TAX_IDENTIFIER_TYPES = [
     'is_vsk',
     'it_iva',
     'jp_cn',
+    'ke_pin',
     'kr_brn',
     'lt_pvm',
     'lu_tva',
@@ -742,6 +754,7 @@ TAX_IDENTIFIER_TYPES = [
     'ma_ice',
     'mc_tva',
     'md_idno',
+    'me_pib',
     'mk_edb',
     'mt_vat',
     'mx_rfc',
@@ -756,11 +769,16 @@ TAX_IDENTIFIER_TYPES = [
     'rs_pib',
     'ru_inn',
     'se_vat',
+    'sg_uen',
     'si_ddv',
     'sk_dph',
     'sm_coe',
+    'sv_nit',
     'th_tin',
     'tn_mf',
+    'tr_vkn',
+    'tr_vkn',
+    'tw_ubn',
     'ua_edrpou',
     'ua_rntrc',
     'us_atin',
@@ -769,7 +787,7 @@ TAX_IDENTIFIER_TYPES = [
     'us_ptin',
     'us_ssn',
     'us_tin',
-    'uy_ruc',
+    'uy_rut',
     've_rif',
     'vn_mst',
     'za_tin',
@@ -819,6 +837,10 @@ class Identifier(sequence_ordered(), DeactivableMixin, ModelSQL, ModelView):
                     [table.type], [new],
                     where=table.type == old))
 
+        # Migration from 7.2: Rename uy_ruc into uy_rut
+        cursor.execute(*table.update([table.type], ['uy_rut'],
+                where=(table.type == 'uy_ruc')))
+
     @classmethod
     def get_types(cls):
         pool = Pool()
