@@ -1,6 +1,9 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import unittest
+from unittest.mock import patch
+
+from stdnum import get_cc_module
 
 try:
     import phonenumbers
@@ -9,7 +12,8 @@ except ImportError:
 
 from trytond.exceptions import UserError
 from trytond.model.exceptions import AccessError
-from trytond.modules.party.party import IDENTIFIER_TYPES
+from trytond.modules.party.party import (
+    IDENTIFIER_TYPES, IDENTIFIER_VAT, replace_vat)
 from trytond.pool import Pool
 from trytond.tests.test_tryton import ModuleTestCase, with_transaction
 from trytond.transaction import Transaction
@@ -100,6 +104,102 @@ class PartyTestCase(PartyCheckEraseMixin, ModuleTestCase):
                 'code': code,
                 })
 
+    @with_transaction()
+    def test_party_autocomplete_eu_vat(self):
+        "Test party autocomplete eu_vat"
+        pool = Pool()
+        Party = pool.get('party.party')
+
+        self.assertEqual(
+            Party.autocomplete('BE500923836'), [{
+                    'id': None,
+                    'name': 'BE0500923836',
+                    'defaults': {
+                        'identifiers': [{
+                                'type': 'eu_vat',
+                                'code': 'BE0500923836',
+                                }],
+                        },
+                    }])
+
+    @with_transaction()
+    def test_party_autocomplete_eu_vat_without_country(self):
+        "Test party autocomplete eu_vat without country"
+        pool = Pool()
+        Party = pool.get('party.party')
+
+        self.assertIn({
+                'id': None,
+                'name': 'BE0500923836',
+                'defaults': {
+                    'identifiers': [{
+                            'type': 'eu_vat',
+                            'code': 'BE0500923836',
+                            }],
+                    },
+                },
+            Party.autocomplete('500923836'))
+
+    @with_transaction()
+    def test_party_autocomplete_be_vat(self):
+        "Test party autocomplete be_vat"
+        pool = Pool()
+        Party = pool.get('party.party')
+        Configuration = pool.get('party.configuration')
+
+        configuration = Configuration(1)
+        configuration.identifier_types = ['be_vat']
+        configuration.save()
+
+        self.assertEqual(
+            Party.autocomplete('BE500923836'), [{
+                    'id': None,
+                    'name': '0500923836',
+                    'defaults': {
+                        'identifiers': [{
+                                'type': 'be_vat',
+                                'code': '0500923836',
+                                }],
+                        },
+                    }])
+
+    @with_transaction()
+    def test_party_default_get_eu_vat(self):
+        "Test party default_get eu_vat"
+        pool = Pool()
+        Party = pool.get('party.party')
+        Country = pool.get('country.country')
+
+        belgium = Country(code='BE', name="Belgium")
+        belgium.save()
+
+        eu_vat = get_cc_module('eu', 'vat')
+        with patch.object(eu_vat, 'check_vies') as check_vies:
+            check_vies.return_value = {
+                'valid': True,
+                'name': "Tryton Foundation",
+                'address': "Street",
+                }
+            with Transaction().set_context(default_identifiers=[{
+                            'type': 'eu_vat',
+                            'code': 'BE0500923836',
+                            }]):
+                self.assertEqual(
+                    Party.default_get(['name', 'addresses', 'identifiers']), {
+                        'name': "Tryton Foundation",
+                        'addresses': [{
+                                'street': "Street",
+                                'country': belgium.id,
+                                'country.': {
+                                    'rec_name': "🇧🇪 Belgium",
+                                    },
+                                }],
+                        'identifiers': [{
+                                'type': 'eu_vat',
+                                'code': 'BE0500923836',
+                                }],
+                        })
+
     @with_transaction()
     def test_address(self):
         'Create address'
@@ -126,6 +226,102 @@ class PartyTestCase(PartyCheckEraseMixin, ModuleTestCase):
                 "St sample, 15\n"
                 "City")
 
+    @with_transaction()
+    def test_address_autocomplete_postal_code(self):
+        "Test autocomplete of postal code"
+        pool = Pool()
+        Country = pool.get('country.country')
+        Subdivision = pool.get('country.subdivision')
+        PostalCode = pool.get('country.postal_code')
+        Address = pool.get('party.address')
+
+        country = Country(name="Country")
+        country.save()
+        subdivision = Subdivision(name="Subdivision", country=country)
+        subdivision.save()
+        postal_code = PostalCode(
+            country=country,
+            subdivision=subdivision,
+            postal_code="12345",
+            city="City",
+            )
+        postal_code.save()
+
+        address = Address(
+            country=country, subdivision=subdivision, city="C")
+        completions = address.autocomplete_postal_code()
+
+        self.assertEqual(completions, ["12345"])
+
+    @with_transaction()
+    def test_address_autocomplete_postal_code_empty(self):
+        "Test autocomplete with empty postal code"
+        pool = Pool()
+        Country = pool.get('country.country')
+        PostalCode = pool.get('country.postal_code')
+        Address = pool.get('party.address')
+
+        country = Country(name="Country")
+        country.save()
+        for postal_code in [None, '12345']:
+            PostalCode(
+                country=country,
+                postal_code=postal_code,
+                city="City",
+                ).save()
+
+        completions = Address(city="City").autocomplete_postal_code()
+
+        self.assertEqual(completions, ["12345"])
+
+    @with_transaction()
+    def test_address_autocomplete_city(self):
+        "Test autocomplete of city"
+        pool = Pool()
+        Country = pool.get('country.country')
+        Subdivision = pool.get('country.subdivision')
+        PostalCode = pool.get('country.postal_code')
+        Address = pool.get('party.address')
+
+        country = Country(name="Country")
+        country.save()
+        subdivision = Subdivision(name="Subdivision", country=country)
+        subdivision.save()
+        postal_code = PostalCode(
+            country=country,
+            subdivision=subdivision,
+            postal_code="12345",
+            city="City",
+            )
+        postal_code.save()
+
+        address = Address(
+            country=country, subdivision=subdivision, postal_code="123")
+        completions = address.autocomplete_city()
+
+        self.assertEqual(completions, ["City"])
+
+    @with_transaction()
+    def test_address_autocomplete_city_empty(self):
+        "Test autocomplete with empty city"
+        pool = Pool()
+        Country = pool.get('country.country')
+        PostalCode = pool.get('country.postal_code')
+        Address = pool.get('party.address')
+
+        country = Country(name="Country")
+        country.save()
+        for city in [None, "City"]:
+            PostalCode(
+                country=country,
+                postal_code="12345",
+                city=city,
+                ).save()
+
+        completions = Address(postal_code="12345").autocomplete_city()
+
+        self.assertEqual(completions, ["City"])
+
     @with_transaction()
     def test_full_address_country_subdivision(self):
         'Test full address with country and subdivision'
@@ -466,9 +662,16 @@ class PartyTestCase(PartyCheckEraseMixin, ModuleTestCase):
         "Ensure tax identifier types are in identifier types"
         pool = Pool()
         Party = pool.get('party.party')
-        self.assertFalse(
-            set(Party.tax_identifier_types())
-            - set(dict(IDENTIFIER_TYPES).keys()))
+        identifiers = dict(IDENTIFIER_TYPES).keys()
+        tax_identifiers = set(Party.tax_identifier_types())
+        self.assertLessEqual(tax_identifiers, identifiers)
+
+    @with_transaction()
+    def test_identifier_vat_types(self):
+        "Ensure VAT identifiers are identifier types"
+        identifiers = dict(IDENTIFIER_TYPES).keys()
+        vat_identifiers = set(map(replace_vat, IDENTIFIER_VAT))
+        self.assertLessEqual(vat_identifiers, identifiers)
 
     @with_transaction()
     def test_party_distance(self):
